/*
 * i2c_slave.c
 *
 *  Created on: 13 Feb 2018
 *      Author: ctorr
 */


// MULTOS running in Embedded mode
// Demonstrates using high level I2C calls to talk to an Arduino with the Arduino as the master
// To run, load the application first (in Command Mode) then reset the power in Embedded mode

#pragma attribute("aid", "F4000005")
#pragma attribute("access_list", "1003") // Includes "Strong Cryptography" bit
#pragma attribute("name", "I2C Slave")

#include <multos.h>
#include <mi.h>
#include <stdio.h>
#include <string.h>

// I2C commands
enum cmd_t {INCREMENT=0, ENCRYPT};
#define MAX_CMD_DATA 64
typedef struct {
	BYTE cmd;
	BYTE len;
	BYTE data[MAX_CMD_DATA];
} input_t;

#pragma melstatic
MI_I2C_SLAVE_CONFIG config = {	0x00,	//address mode = 7bit
								0x0010,	//address
								0x0104,	//rxBufferSize = 260 bytes
								0x00,	//bus Speed (fixed RFU)
								0x01,	//frameStartHandlingEnabled (fixed RFU)
								0x01,	//clockStretchingEnabled
								0x01,	//stopOnCollision
								0x1400	//port Config SDA=GPIO 1.1, SCL=GPIO 1.0
};

#pragma melperso
// This key would be personalised for each device
BYTE aesKey[] = { 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7, 0xA8 };

#pragma melsession
input_t command;

void abendFnc(void)
{
	BYTE code;

	// Get abend reason code
	code = miAbendReason();

	// Output abend details on serial port
	printf("Abend code %d\n",code);

	miPerformRestart();
}

void
main(void)
{
	BYTE id;
	BYTE i;
	BYTE n;

	// The command arrives on the serial port
	// If starting up for the first time
	id = miGetEmbeddedEventId();
	if( id == MI_EMBEDDED_EVENT_STARTUP)
	{
		// Register abend handler
		miSetAbendFunction((WORD)abendFnc);

		// Set up I2C
		miI2CSlaveInitialise (&config);

		// Set up serial port to output messages
		miSerialIOInitialise(MI_PACKAGE_PIN_27_GPIO_PIN_NUMBER,MI_PACKAGE_PIN_26_GPIO_PIN_NUMBER,9600,MI_SERIAL_IO_8_BITS_NO_PARITY_1_STOP_BIT,100);

		// Register for I2C events
		miRegisterEmbeddedEvent (AM_EMBEDDED_EVENT_I2C_SLAVE, 0);

		return;
	}

	// Should only get to here if an I2C event has occurred

	// Read I2C data
	miI2CSlaveGetReceivedBytes((BYTE *)&command,sizeof(command));
	printf("Received command %d len %d\n",command.cmd, command.len);
	for(i=0;i<command.len;i++)
		printf("%02x ",command.data[i]);
	printf("\n");

	// Do the requested command
	switch(command.cmd)
	{
		case INCREMENT: // Add one to all bytes in the data
			for(i=0;i<command.len && i < MAX_CMD_DATA;i++)
				command.data[i] ++;
			break;

		case ENCRYPT: // AES-128 encrypt the provided data.
			// Pad the provided data upto the block size of 16 bytes for AES
			// Calculate the number of padding bytes needed
			n = command.len % 16;

			// Do the padding
			for(i = 0; i < n; i++)
				command.data[command.len+i] = 0;
			command.len += n;

			// Do the encryption
			multosEncipherECB(ALGORITHM_AES,command.data,command.data,sizeof(aesKey),aesKey,command.len);
			break;

		default:
			memset(command.data,0xCC,4);
			command.len = 4;
			break;
	}

	// Output result
	miI2CSlaveTransmitBytes((BYTE *)&command,2+command.len);
}
