/*
 * led_flash.c
 *
 *  Created on: 12 Feb 2018
 *      Author: ctorr
 */

// Runs in COMMAND MODE:
// This application will flash the on-board LED. The number of times to flash is given in the
// command P1 parameter and the gap between flashes (in seconds) by the P2 parameter.
// The structure of an APDU is described in the MULTOS Developers Guide http://www.multos.com/uploads/MDG.pdf

// Application Identifier - used to select the application
#pragma attribute("aid","F4000001")
// The name that will appear in the Directory entry
#pragma attribute("name","LED Flash")
// Permission to use peripherals (bit12), strong crypto (bit0) and the ISO contact interface (bit1)
#pragma attribute("access_list","1003")

// Standard MULTOS include file
#include <multos.h>

// Include file for Trust Anchor specific functions
#include <mi.h>

// Include file for ISO defined status values
#include <ISO7816.h>
#define ISO_CASE_NO_DATA	1
#define ISO_CASE_DATA_OUT	2
#define ISO_CASE_DATA_IN	3
#define ISO_CASE_DATA_IN_OUT 4

// Define GPIO pin that the LED is connected to
#define LED_PIN	6

// Define supported APDU Class and Instruction bytes
#define SUPPORTED_CLA	0x70
#define FLASH_INS		0x00

// Declare stored variables. These are saved in the flash memory of the chip.
#pragma melstatic
BYTE lastError;

// Declare temporary variables. These are created (and initialised to zero) when the application is selected
// and cleared when the chip is reset or the application is deselected.
#pragma melsession
static BYTE initialised;

// Exception handler
void
abendHandler(void)
{
	lastError = miAbendReason();
	miPerformRestart();
}

void
main(void)
{
	BYTE i;

	// Validate APDU
	if(!multosCheckCase(ISO_CASE_NO_DATA))
		multosExitSW(ISO7816_SW_WRONG_PARAMS);
	if(CLA != SUPPORTED_CLA)
		multosExitSW(ISO7816_SW_CLA_NOT_SUPPORTED);
	if(INS != FLASH_INS)
		multosExitSW(ISO7816_SW_INS_NOT_SUPPORTED);

	// If built and executed from Eclipse, debugging will be enabled
	// If built from build_run.bat then the application will run normally.
#ifndef NO_DEBUG
	miDebuggingEnable();
#endif

	// Initialisation of the peripherals is only required once per session. Multiple calls will result in an exception (abend).
	miSetAbendFunction((WORD)abendHandler);
	if(!initialised)
	{
		// Allocate GPIO pin 6 (the one connected to the on-board LED) and set it as an Output.
		miGPIOAllocatePin(LED_PIN);
		miGPIOEnableOutputPin(LED_PIN);
		miGPIOSetPin(LED_PIN,MI_GPIO_PIN_STATE_LOW);

		initialised = 1;
	}

	// Flash the LED P1 number of times
	for(i = 0; i < P1; i++)
	{
		miGPIOSetPin(LED_PIN,MI_GPIO_PIN_STATE_HIGH);
		miDelay(P2*1000);
		miGPIOSetPin(LED_PIN,MI_GPIO_PIN_STATE_LOW);
		miDelay(P2*1000);
	}

	// Application will exit with the default ISO status word of 0x9000
	// unless otherwise stated.
}
